<?php

namespace Drupal\search_api_meilisearch\Api;

use MeiliSearch\Client;
use MeiliSearch\Exceptions\HTTPRequestException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class MeiliSearchApiService.
 */
class MeiliSearchApiService implements MeiliSearchApiServiceInterface {

  use StringTranslationTrait;

  /**
   * The ConfigFactory instance.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * MeiliSearch client class.
   *
   * @var \MeiliSearch\Client
   */
  protected $client;

  /**
   * The construct method for the MeiliSearchApiService.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The ConfigFactoryInterface instance.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
    $config = $this->configFactory->get('search_api_meilisearch.settings');
    $host_address = $config->get('meilisearch_host_address');
    $port = $config->get('meilisearch_host_port');
    $private_key = $config->get('meilisearch_private_key');
    $url = $host_address . ':' . $port;
    $this->client = new Client($url, $private_key);
  }

  /**
   * {@inheritdoc}
   */
  public function search($index_name, $query, array $options = []) {
    if ($index_name == '' || $query == '') {
      return NULL;
    }
    try {
      $index = $this->getIndex($index_name);
      if ($index) {
        return $index->search($query, $options);
      }
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getIndex($name) {
    try {
      return $this->client->getIndex($name);
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createIndex($attributes) {
    try {
      $this->client->createIndex($attributes);
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function removeIndex($index_name) {
    try {
      $this->client->deleteIndex($index_name);
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function listIndexes() {
    try {
      return $this->client->getAllIndexes();
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function addDocuments($index_name, array $documents = []) {
    try {
      $index = $this->getIndex($index_name);
      $index->addDocuments($documents);
      return $index->getDocuments();
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDocument($index_name, $id) {
    try {
      $index = $this->getIndex($index_name);
      return $index->getDocument($id);
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getDocuments($index_name, array $options = []) {
    try {
      $index = $this->getIndex($index_name);
      $documents = $index->getDocuments($options);
      if (empty($documents)) {
        return NULL;
      }
      return $documents;
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDocument($index_name, $id) {
    if ($index_name) {
      try {
        $index = $this->getIndex($index_name);
        $index->deleteDocument($id);
        return $this->getDocuments($index_name);
      }
      catch (HTTPRequestException $e) {
        throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
      }
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDocuments($index_name, $ids) {
    try {
      $index = $this->getIndex($index_name);
      $index->deleteDocuments($ids);
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAllDocuments($index_name) {
    try {
      $index = $this->getIndex($index_name);
      $index->deleteAllDocuments();
    }
    catch (HTTPRequestException $e) {
      throw new MeiliSearchApiException($e->getMessage(), $e->getCode(), $e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function prepareItems(array $items) {
    $prepared_items = [];
    $config = $this->configFactory->getEditable('search_api_meilisearch.settings');
    $id_mapping = [];
    foreach ($items as $item) {
      $item_id = str_replace([':', '/'], '_', $item->getId());
      $id_mapping[$item->getIndex()->id()][$item_id] = $item->getId();
      $item_to_index = ['id' => $item_id];
      $item_fields = $item->getFields();

      foreach ($item_fields as $field) {
        $type = $field->getType();
        $values = NULL;
        $field_values = $field->getValues();

        if (empty($field_values)) {
          continue;
        }

        foreach ($field_values as $field_value) {
          if (!$field_value) {
            continue;
          }
          switch ($type) {
            case 'text':
            case 'string':
            case 'uri':
              $field_value .= '';
              if (mb_strlen($field_value) > 10000) {
                $field_value = mb_substr(trim($field_value), 0, 10000);
              }
              $values[] = $field_value;
              break;

            case 'integer':
            case 'duration':
            case 'decimal':
              $values[] = 0 + $field_value;
              break;

            case 'boolean':
              $values[] = $field_value ? TRUE : FALSE;
              break;

            case 'date':
              if (is_numeric($field_value) || !$field_value) {
                $values[] = 0 + $field_value;
                break;
              }
              $values[] = strtotime($field_value);
              break;

            default:
              $values[] = $field_value;
          }
        }
        if (is_array($values) && count($values) <= 1) {
          $values = reset($values);
        }
        $item_to_index[$field->getFieldIdentifier()] = $values;
      }
      $prepared_items[] = $item_to_index;
    }
    // Map drupal ids to ids prepared for MeiliSearch.
    $config->set('meilisearch_id_mappings', $id_mapping)->save();
    return $prepared_items;
  }

}
