<?php

namespace Drupal\search_api_meilisearch\Api;

/**
 * Interface MeiliSearchApiServiceInterface.
 */
interface MeiliSearchApiServiceInterface {

  /**
   * Searches the string on the specified index.
   *
   * @param string $index
   *   The name of the index to perform the search on.
   * @param string $query
   *   A query string to perform the search on.
   * @param array $options
   *   An array of options for MeiliSearch API.
   *
   * @return array
   *   An array of search results.
   *
   * @see https://docs.meilisearch.com/references/search.html#search-in-an-index
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function search($index, $query, array $options = []);

  /**
   * Gets the index information.
   *
   * @param string $index_name
   *   The name of the index.
   *
   * @return array
   *   Returns the the index information.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function getIndex($index_name);

  /**
   * Creates an index with attributes specified.
   *
   * @param string|array $attributes
   *   The name of the index or an array of attributes.
   *
   * @see https://github.com/meilisearch/meilisearch-php/
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function createIndex($attributes);

  /**
   * Removes an index with the specified name.
   *
   * @param string $name
   *   The name of the index to remove.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function removeIndex($name);

  /**
   * Lists all indexes.
   *
   * @return array
   *   Data with all the indexes.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function listIndexes();

  /**
   * Adds documents to the index.
   *
   * @param string $index
   *   The name of the index.
   * @param array $documents
   *   An array of arrays of documents you want to add to index.
   *
   * @return array
   *   An array of all documents in the specified index.
   *
   * @see https://github.com/meilisearch/meilisearch-php/
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function addDocuments($index, array $documents);

  /**
   * Gets a specific document from an index.
   *
   * @param string $index
   *   The name of the index.
   * @param string $id
   *   An id of the document.
   *
   * @return array
   *   Document data.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function getDocument($index, $id);

  /**
   * Retrieves multiple documents from an index.
   *
   * @param string $index
   *   The name of the index.
   * @param array $options
   *   Array of options to get the documents by.
   *
   * @example $options = ['offset' => 10 , 'limit' => 20]
   *
   * @return array
   *   List of documents from specified index.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function getDocuments($index, array $options);

  /**
   * Deletes a single document.
   *
   * @param string $index
   *   The name of the index.
   * @param string $id
   *   The id of the document to delete.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function deleteDocument($index, $id);

  /**
   * Deletes multiple documents.
   *
   * @param string $index_name
   *   The name of the index.
   * @param array $ids
   *   An array of document's ids.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function deleteDocuments($index_name, array $ids);

  /**
   * Deletes all documents for a specific index.
   *
   * @param string $index_name
   *   The name of the index from which to delete documents.
   *
   * @throws \Drupal\search_api_meilisearch\Api\MeiliSearchApiException
   *   Exception on an API call failure.
   */
  public function deleteAllDocuments($index_name);

  /**
   * Prepares items for indexing on a MeiliSearch server.
   *
   * @param array $items
   *   Items that needs to be indexed.
   */
  public function prepareItems(array $items);

}
