<?php

namespace Drupal\search_api_meilisearch\Plugin\search_api\backend;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\search_api_meilisearch\Api\MeiliSearchApiException;
use Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface;
use Drupal\search_api\Backend\BackendPluginBase;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Query\QueryInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Class SearchApiMeiliSearchBackend.
 *
 * @SearchApiBackend(
 *   id = "search_api_meilisearch",
 *   label = @Translation("MeiliSearch"),
 *   description = @Translation("Index items using MeiliSearch Server.")
 * )
 */
class SearchApiMeiliSearchBackend extends BackendPluginBase implements PluginFormInterface {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * MeiliSearch service for making API calls.
   *
   * @var \Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface
   */
  protected $meiliService;

  /**
   * The logger to use for logging messages.
   *
   * @var \Psr\Log\LoggerInterface|null
   */
  protected $logger;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The construct method for the MeiliSearch back-end plugin.
   *
   * @param array $configuration
   *   Configuration array.
   * @param string $plugin_id
   *   The plugin id.
   * @param mixed $plugin_definition
   *   A plugin definition.
   * @param \Drupal\search_api_meilisearch\Api\MeiliSearchApiServiceInterface $meiliService
   *   A MeiliSearchApiService instance.
   * @param \Psr\Log\LoggerInterface $logger
   *   A LoggerInterface instance.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   A LanguageManagerInterface instance.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   A ConfigFactory instance.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MeiliSearchApiServiceInterface $meiliService, LoggerInterface $logger, LanguageManagerInterface $languageManager, ConfigFactoryInterface $configFactory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->meiliService = $meiliService;
    $this->logger = $logger;
    $this->languageManager = $languageManager;
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('search_api_meilisearch.api'),
      $container->get('logger.channel.search_api_meilisearch'),
      $container->get('language_manager'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewSettings() {
    $indexes_list = [];
    try {
      // Available indexes.
      $indexes = $this->meiliService->listIndexes();
      if (isset($indexes)) {
        foreach ($indexes as $index) {
          $indexes_list[] = $index['name'];
        }
      }
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('Could not list indexes.'));
    }

    $info[] = [
      'label' => $this->t('Available MeiliSearch indexes'),
      'info' => !empty(implode(', ', $indexes_list)) ? implode(', ', $indexes_list) : $this->t('No indexes found.'),
    ];
    return $info;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('search_api_meilisearch.settings');

    $form['meilisearch_host_address'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Host Address:'),
      '#maxlength' => 255,
      '#size' => 30,
      '#required' => TRUE,
      '#description' => $this->t('The host address of the MeiliSearch server.'),
      '#default_value' => $config->get('meilisearch_host_address') != '' ? $config->get('meilisearch_host_address') : 'http://127.0.0.1',
      '#attributes' => [
        'placeholder' => 'http://127.0.0.1',
      ],
    ];

    $form['meilisearch_host_port'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Host Port:'),
      '#maxlength' => 5,
      '#size' => 5,
      '#required' => TRUE,
      '#description' => $this->t('The port of the MeiliSearch server.'),
      '#attributes' => [
        'placeholder' => '7700',
      ],
      '#default_value' => $config->get('meilisearch_host_port') != '' ? $config->get('meilisearch_host_port') : '7700',
    ];

    $form['meilisearch_private_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeiliSearch Private Key:'),
      '#maxlength' => 255,
      '#size' => 64,
      '#default_value' => $config->get('meilisearch_private_key'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $host_port = (int) $form_state->getValue('meilisearch_host_port');

    if ($host_port > 65535 || $host_port <= 0) {
      $form_state->setErrorByName('meilisearch_host_port', $this->t('Host port must be between 1 and 65535'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('search_api_meilisearch.settings');
    $hostName = $form_state->getvalue('meilisearch_host_address');
    $port = $form_state->getvalue('meilisearch_host_port');
    $privateKey = $form_state->getvalue('meilisearch_private_key');
    $config->set('meilisearch_host_address', $hostName);
    $config->set('meilisearch_host_port', $port);
    $config->set('meilisearch_private_key', $privateKey);
    $config->save();
  }

  /**
   * {@inheritdoc}
   */
  public function removeIndex($index) {
    try {
      $this->meiliService->removeIndex($index->id());
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('Could not remove @name index.', ['@name' => $index->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function indexItems(IndexInterface $index, array $items) {
    try {
      $documents = $this->meiliService->prepareItems($items);
      $this->meiliService->addDocuments($index->id(), $documents);
      return array_keys($items);
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('Could not index items for @name index.', ['@name' => $index->id()]));
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function addIndex(IndexInterface $index) {
    try {
      $this->meiliService->createIndex($index->id());
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('Index @name could not be created.', ['@name' => $index->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteItems(IndexInterface $index, array $item_ids) {
    try {
      $ids = [];
      foreach ($item_ids as $id) {
        $item_id = str_replace([':', '/'], '_', $id);
        $ids[] = $item_id;
      }
      $this->meiliService->deleteDocuments($index->id(), $ids);
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('The documents for index @name could not be deleted.', ['@name' => $index->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAllIndexItems(IndexInterface $index, $datasource_id = NULL) {
    try {
      $this->meiliService->deleteAllDocuments($index->id());
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('The documents for index @name could not be deleted.', ['@name' => $index->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function search(QueryInterface $query) {
    $results = $query->getResults();
    $keys = $query->getOriginalKeys();
    try {
      $data = $this->meiliService->search($query->getIndex()->id(), $keys);

      // If the query is empty we return all documents.
      // This is necessary for the server index status item's count.
      if ($data == NULL) {
        $data['hits'] = $this->meiliService->getDocuments($query->getIndex()->id(), []);
      }

      $config = $this->configFactory->getEditable('search_api_meilisearch.settings');
      $id_maping_config = $config->get('meilisearch_id_mappings');

      if (empty($data['hits'])) {
        return $results;
      }

      if (isset($id_maping_config) && array_key_exists($query->getIndex()->id(), $id_maping_config)) {
        $id_mapping = $config->get('meilisearch_id_mappings')[$query->getIndex()->id()];

        $results->setResultCount(count($data['hits']));
        foreach ($data['hits'] as $row) {
          if (array_key_exists($row['id'], $id_mapping)) {
            $result_id = $id_mapping[$row['id']];
          }
          $item = $this->getFieldsHelper()
            ->createItem($query->getIndex(), $result_id);
          $results->addResultItem($item);
        }
      }
      return $results;
    }
    catch (MeiliSearchApiException $e) {
      $this->logger->warning($e->getMessage());
      $this->messenger()->addWarning($this->t('Could not connect to the MeiliSearch server.'));
    }
    return $results;
  }

}
